/*
	This file is part of lobj.

	lobj is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lobj is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lobj.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "list.h"

/**
 * Gets the size of a table.
 * 
 * @param L lua_State to use
 * @param index The index of the table on the Lua stack
 * @return Size of the table
 */
static int priv_get_table_size(lua_State* L, int index)
{
	int stack_size = lua_gettop(L);
	lua_pushstring(L, "table");
	lua_rawget(L, LUA_GLOBALSINDEX);
	lua_pushstring(L, "getn");
	lua_rawget(L, -2);
	lua_pushvalue(L, index);
	lua_call(L, 1, 1);
	int table_size = lua_tonumber(L, -1);
	lua_settop(L, stack_size);
	return table_size;
}

int lobj_gstrv2table(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TLIGHTUSERDATA);
	gsize length = G_MAXSIZE;
	
	if(lua_gettop(L) == 2)
	{
		luaL_checktype(L, 2, LUA_TNUMBER);
		length = lua_tointeger(L, 2);
	}
	
	gchar* value = NULL;
	GStrv strv = lua_touserdata(L, 1);
	int i;

	lua_newtable(L);

	for(i = 0; i < length; ++i)
	{
		value = strv[i];

		#ifdef IDEBUG
		g_debug("Iterating: i == %i and value == %s", i, value);
		#endif

		if(value == NULL) break;

		/* Push the key and them the value */
		lua_pushnumber(L, i + 1);
		lua_pushstring(L, value);
		lua_rawset(L, -3);
	}
	
	#ifdef IDEBUG
	g_debug("Returning a GStrv with %i elements!", i);
	#endif
	
	return 1;
}

int lobj_table2gstrv(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TTABLE);
	
	int i = 0;
	int table_size = priv_get_table_size(L, 1);

	/* Allocate the array, for table_size elements + 1 NULL */
	GStrv array = g_malloc(sizeof(gchar*) * (table_size + 1));

	/*
	 * Transport the values to a array
	 */
	for (;;)
	{
		lua_rawgeti(L, 1, ++i);

		#ifdef IDEBUG
		g_debug("Iterating: i == %i, type == %s", i, lua_typename(L, lua_type(L, -1)));
		#endif

		if(lua_isnil(L, -1)) break;

		gchar* tmp = g_strdup(lua_tostring(L, -1));
		array[i - 1] = tmp;
      	lua_pop(L, 1);
    }
    lua_pop(L, 1);

    /* Set the guard */
    array[i - 1] = NULL;

	lua_pushlightuserdata(L, array);
	return 1;
}

int lobj_gslist2table(lua_State* L)
{
	int i = 0;
	GSList* list = lua_touserdata(L, 1);
	Type type = lua_tointeger(L, 2); 

	lua_newtable(L);

	while(list)
	{
		switch(type)
		{
			case TYPE_STRING: lua_pushstring(L, (const gchar*)list->data); break;
			case TYPE_DOUBLE: lua_pushnumber(L, *(double*)list->data); break;
			case TYPE_OBJECT: priv_object_new(L, list->data, FALSE); break;
			case TYPE_CUSTOM: /* Passed special handler, that must be on the top */
			{
				lua_pushvalue(L, -2); /* Copy the function, to reuse it */
				lua_pushlightuserdata(L, list->data);				
				lua_call(L, 1, 1); /* This leaves the desired value on the top */
				break;
			}
			default: priv_struct_new(L, list->data, FALSE); break;
		}

		lua_rawseti(L, -2, ++i);
		list = list->next;
	}

	#ifdef IDEBUG
	g_debug("Returning a table with %i elements!", i);
	#endif
	
	return 1;
}

int lobj_glist2table(lua_State* L)
{
	int i = 0;
	GList* list = lua_touserdata(L, 1);
	Type type = lua_tointeger(L, 2);

	lua_newtable(L);

	while(list)
	{
		switch(type)
		{
			case TYPE_STRING: lua_pushstring(L, (const gchar*)list->data); break;
			case TYPE_DOUBLE: lua_pushnumber(L, *(double*)list->data); break;
			case TYPE_OBJECT: priv_object_new(L, list->data, FALSE); break;
			case TYPE_CUSTOM: /* Passed special handler, that must be on the top */
			{
				lua_pushvalue(L, -2); /* Copy the function, to reuse it */
				lua_pushlightuserdata(L, list->data);				
				lua_call(L, 1, 1); /* This leaves the desired value on the top */
				break;
			}
			
			default: priv_struct_new(L, list->data, FALSE); break;
		}		

		lua_rawseti(L, -2, ++i);
		list = list->next;
	}

	#ifdef IDEBUG
	g_debug("Returning a table with %i elements!", i);
	#endif
	
	return 1;
}
